require 'uri'

module RD
  class RD2RWikiVisitor < RDVisitor

    def ref_ext_RubyML(label, content)
      return nil unless /^(ruby-(?:talk|list|dev|math|ext)):\s*(\d+)$/ =~ label
      ml = $1
      article = $2.sub(/^0+/, '')
      content = "[#{label}]" if label == content
      url_ref_ext("http://blade.nagaokaut.ac.jp/cgi-bin/scat.rb/ruby/#{ ml }/#{ article }", content)
    end

    def ref_ext_RAA(label, content)
      return nil unless /^RAA:\s*(.+)$/ =~ label
      name = CGI.escape($1)
      content = "[#{label}]" if label == content
      url_ref_ext("http://www.ruby-lang.org/en/raa-list.rhtml?name=#{ name }", content)
    end

    ALLOWED_IMG_URL_SCHEME = ['http', 'https']
    def ref_ext_img(label, content)
      return nil unless /^img:\s*(.+)$/ =~ label
      url = $1
      begin
	scheme = URI.parse(url).scheme.to_s.downcase
	return nil unless ALLOWED_IMG_URL_SCHEME.include?(scheme)
      rescue
	return nil
      end
      resource = CGI.escapeHTML(url)
      content = CGI.escapeHTML(content)
      if label == content
	%Q[<img src="#{resource}" />]
      else
	%Q[<img src="#{resource}" alt="#{content}" />]
      end
    end

    RUBY_MAN_BASE = "http://www.ruby-lang.org/ja/man-1.6/"
    def ref_ext_rubyman(label, content)
      return nil unless /^rubyman-1.6:\s*([^#]+)(?:#(.+))?$/ =~ label
      name, part = $1, $2
      name = CGI.escape(name)
      part = CGI.escape(part) if part
      content = "[#{label}]" if label == content
      url = RUBY_MAN_BASE + "?cmd=view;name=" + name
      url << '#' << part if part
      url_ref_ext(url, content)
    end

    def ext_VerbatimBlock_RT(content)
      require 'rt/rtparser'
      require 'rt/rt2html-lib'

      @rt_visitor ||= RT::RT2HTMLVisitor.new
      @rt_visitor.visit(RT::RTParser::parse(content))
    rescue LoadError
      return nil
    end

    # Add here!

  end # RD2RWikiVisitor
end # RD
